#!/usr/bin/perl

use strict;
use Getopt::Long;
use POSIX;
use Date::Manip;

#=============================================================================#

my $opt_help;
my $opt_version;
my $opt_list;
my $opt_add;
my $opt_del;
my $opt_unlink;
my $opt_from;
my $opt_to;
my $version;
my $mapdir;
my %FILE;
my %USER;
my %CTIME;

#-----------------------------------------------------------------------------#

sub dirCheck()
  {
    -d $mapdir || die "No $mapdir on this machine: $!\n";
  }

#-----------------------------------------------------------------------------#

sub dirStat($)
  {
    my $dir = shift;

    my $file;
    my $ino;
    my $ctime;
    my @Files;
    my @User;
    my @Url;

    opendir DIR, $dir or die $!;
    @Files = grep !/\.\.?$/, readdir DIR;
    closedir DIR;

    @User = grep !/%/, @Files;
    @Url  = grep  /%/, @Files;

    foreach $file (@User)
      {
	$ino = (stat("$mapdir/$file"))[1];

	unless (exists $FILE{$ino})
	  {
	    $FILE{$ino} = $file;
	  }
      }

    foreach $file (@Url)
      {
	($ino, $ctime) = (stat("$mapdir/$file"))[1, 10];

	if (exists $FILE{$ino})
	  {
	    $USER{$FILE{$ino}} = $file;
	    $CTIME{$FILE{$ino}} = $ctime;
	  }
      }
  }

#-----------------------------------------------------------------------------#

sub CheckTime($)
  {
    my $date = shift;

    my $flag1;
    my $flag2;

    $flag1 = $opt_from ? Date_Cmp($date, $opt_from) : 1;
    $flag2 = $opt_to   ? Date_Cmp($opt_to,   $date) : 1;

    return(($flag1*$flag2 >= 0) ? 1 : 0);
  }

#-----------------------------------------------------------------------------#

sub UserList()
  {
    my $username;
    my $date;
    my $subject;

    foreach $username (sort {$CTIME{$a} <=> $CTIME{$b}} keys %USER)
      {
	chop($date = POSIX::ctime($CTIME{$username}));

	$date = UnixDate($date, '%b %e %H:%M:%S %Y');

	if (CheckTime($date))
	  {
	    ($subject = "$USER{$username}")
	      =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/eg;

	    print "\033[1m$username\033[0m:\n";
	    print "  date    => \"$date\"\n";
	    print "  subject => \"$subject\"\n";
	    print "\n";
	  }
      }
  }

#-----------------------------------------------------------------------------#

sub UserAdd()
  {
    my $username;

    while ($username = getpwent)
      {
	if ($username =~ /^\D+\d+/)
	  {
	    if (-e "$mapdir/$username")
	      {
		print "Found $username\n";
	      }
	    else
	      {
		if (touch("$mapdir/$username"))
		  {
		    print "Added $username\n";
		  }
		else
		  {
		    die "Can't create $mapdir/$username: $!\n";
		  }
	      }
	  }
      }
  }

#-----------------------------------------------------------------------------#

sub UserUnlink()
  {
    my $username;
    my $date;
    my $subject;

    foreach $username (sort keys %USER)
      {
	chop($date = POSIX::ctime($CTIME{$username}));

	if (CheckTime($date))
	  {
	    ($subject = "$USER{$username}")
	      =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/eg;

	    unlink "$mapdir/$USER{$username}";
	    print "Deleted \"$subject\"\n";
	  }
      }
  }

#-----------------------------------------------------------------------------#

sub UserDel()
  {
    my $username;

    UserUnlink;
    foreach $username (sort keys %FILE)
      {
	unlink "$mapdir/$FILE{$username}";
	print "Deleted $FILE{$username}\n";
      }
  }

#-----------------------------------------------------------------------------#

sub touch($)
  {
    my $file = shift;

    open (FH, ">$file") && close FH;
  }


#-----------------------------------------------------------------------------#

sub printHelp()
  {
    print "edg-mkgridpool --help\n";
    print "edg-mkgridpool --version\n";
    print "edg-mkgridpool --add\n";
    print "edg-mkgridpool --del\n";
    print "edg-mkgridpool --unlink [--from=<date>] [--to=<date>]\n";
    print "edg-mkgridpool --list   [--from=<date>] [--to=<date>]\n";
  }

#-----------------------------------------------------------------------------#

sub printVersion()
  {
    print "edg-mkgridpool version $version\n";
  }

#=============================================================================#

$version = '2.9.1';

#-----------------------------------------------------------------------------#

$mapdir = $ENV{GRIDMAPDIR} || '/etc/grid-security/gridmapdir';

#-----------------------------------------------------------------------------#

GetOptions('help|h'    => \$opt_help,
           'version|v' => \$opt_version,
	   'list'      => \$opt_list,
           'add'       => \$opt_add,
	   'del'       => \$opt_del,
	   'unlink'    => \$opt_unlink,
	   'from:s'    => \$opt_from,
	   'to:s'      => \$opt_to);

#-----------------------------------------------------------------------------#

%FILE  = ();
%USER  = ();
%CTIME = ();

#-----------------------------------------------------------------------------#

if ($opt_help)
  {
    printHelp;
  }
elsif ($opt_version)
  {
    printVersion;
  }
elsif ($opt_list)
  {
    dirCheck;
    dirStat($mapdir);
    UserList;
  }
elsif ($opt_add)
  {
    dirCheck;
    UserAdd;
  }
elsif ($opt_del)
  {
    dirCheck;
    dirStat($mapdir);
    UserDel;
  }
elsif ($opt_unlink)
  {
    dirCheck;
    dirStat($mapdir);
    UserUnlink;
  }
else
  {
    printHelp;
  }

#=============================================================================#
